"""
TBC macro to create a simple linestring object
"""
import clr
import types
import math
clr.AddReference('IronPython.Wpf')
import wpf
from System import Math, Action, Func
from System.Text import Encoding
from System.Collections.Generic import List
from System.Windows import Visibility
from System.Windows.Controls import StackPanel
from System.Windows.Input import Keyboard, AccessKeyManager
from System import Type, Tuple, Array, Double
from System.IO import StreamReader, MemoryStream
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys

from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
from Trimble.Vce.SurveyCAD import ProjectLineStyle
from Trimble.DiskIO import SearchPath, OptionsManager

from Trimble.Vce.Alignment.Linestring import *
#Linestring,ElementFactory, IStraightSegment, IXYZLocation,ITangentArcSegment, ArcTangentType
from Trimble.Vce.Alignment.Parameters import Bearing, Elevation

from Trimble.Vce.Data.Construction.Settings import ConstructionCommandsSettings

from Trimble.Vce.Geometry import Point3D

from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.BaseCommands")
from Trimble.Vce.UI.BaseCommands import ViewHelper
clr.AddReference("Trimble.Vce.UI.UIManager")
from Trimble.Vce.UI.UIManager import UIEvents
clr.AddReference("Trimble.Vce.UI.Controls")
from Trimble.Sdk.Interfaces.UI import *
#InputMethod, MousePosition, CursorStyle, ControlBoolean, UIEventArgs
from Trimble.Vce.UI.Controls import VceMessageBox, AngularEdit, BrowseFile






clr.AddReference("Trimble.Sdk.ForeignCad")
from Trimble.Vce.ForeignCad import TextEntity, MText, Text, Leader, Point
clr.AddReference("Trimble.Sdk.Interfaces")
from Trimble.Vce.Interfaces.SnapIn import ISnapIn, IPolyseg, IHaveUcs, IMemberManagement, IName
from Trimble.Vce.Interfaces.Point import IPoint, IPointLocation, IFeatureCodeParent
from Trimble.Vce.Interfaces.Client import CommandGranularity, IProject, IProjectGetter

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "Palar1"
    cmdData.CommandName = "Palar1"
    cmdData.Caption = "_Palar utsättning V1.0"
    cmdData.UIForm = "Palar1"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22055"

    try:
        cmdData.DefaultTabKey = "Macros"
        cmdData.DefaultTabGroupKey = "Trimtec"
        cmdData.ShortCaption = "Pålar utsättning"
        cmdData.DefaultRibbonToolSize = 3 # Default=0, ImageOnly=1, Normal=2, Large=3

        cmdData.Version = 1.0
        cmdData.MacroAuthor = "Trimtec..."
        cmdData.ToolTipTitle = "Pålar utsättning"
        cmdData.ToolTipTextFormatted = "Skapa pålar som tex kan användas vid utsättning"
    except:
        pass
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class Palar1(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):

        if True: # read from file or text
            with StreamReader(macroFileFolder + r"\Palar1.xaml") as s:
                wpf.LoadComponent(self, s)
        else:
            # copy text from *.xaml file and place here
            t = """
            <StackPanel  HorizontalAlignment="Stretch"  Margin="4"
            .... Delete these three lines and cut/paste text from *.xaml file.
            </StackPanel>
            """
            with StreamReader(MemoryStream(Encoding.UTF8.GetBytes(t))) as s:
                wpf.LoadComponent(self, s)

        self.currentProject = currentProject
        self.myLine = 0 # serial number of linestring
        self.times = 5 # Times
        self.nameCtl.AutoTabExtSkip = OptionsManager.GetBool("Palar1:Name", False)
        self.elevCtl.AutoTabExtSkip = OptionsManager.GetBool("Palar1:Elev", True)

        # Punktobjekt
        self.tType = clr.GetClrType(TextEntity)
        self.lType = clr.GetClrType(IPolyseg)
        self.pType = clr.GetClrType(IPoint)
        self.ptType = clr.GetClrType(IPointLocation)
        self.fcType = clr.GetClrType(IFeatureCodeParent)

    def OnLoad(self, cmd, buttons, event):
        # VceMessageBox.Show("OnLoad")
        buttons[0].Content = "Ok"
        buttons[1].Content = "Cancel"
        # buttons[2].Content = "New"
        # buttons[2].Visibility = Visibility.Visible
        self.Caption = cmd.Command.Caption


        # Egna variabler
        self.cordChanged = 0
        self.lutningInst = None
        self.baringInst = None
        self.lutningStatus = "OK"
        self.baringStatus = "OK"

        self.coordCtl.ValueChanged += self.CoordChanged
        buttons[2].Click += self.NewClicked



        ModelEvents.EntityEdited += self.ModelEvents_EntityEdited
        ModelEvents.EntityDeleted += self.ModelEvents_EntityDeleted

        #set default name/layer
        settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
        self.nameCtl.SelectedName = settings.CreateLinestringSettings.Name
        self.Layers.SelectedSerialNumber = settings.CreateLinestringSettings.Layer

        # We want to start cmd with focus on coordinate control.  It's a bit
        # tricky to set focus in the OnLoad method.  The following lines do
        # that.
        def SetFocusToControl():
            Keyboard.Focus(self.coordCtl)
        if self.nameCtl.AutoTabExtSkip:
            Dispatcher.BeginInvoke(Dispatcher.CurrentDispatcher, Action(SetFocusToControl))

    def Dispose(self, thisCmd, disposing):
        # VceMessageBox.Show("Dispose")
        ModelEvents.EntityEdited -= self.ModelEvents_EntityEdited
        ModelEvents.EntityDeleted -= self.ModelEvents_EntityDeleted
        OptionsManager.SetValue("Palar1:Name",         self.nameCtl.AutoTabExtSkip)
        OptionsManager.SetValue("Palar1:Elev",         self.elevCtl.AutoTabExtSkip)
        pass

    def CoordChanged(self, ctrl, e):
        self.cordChanged = 1


    def ModelEvents_EntityEdited(self, obj, eArgs):
        serial = eArgs.EntitySerialNumber
        # check if current line is being edited
        if serial == self.myLine:
            ## EDIT -> STEG1(NY linje)
            ## EDIT -> STEG1(Punkt2 linje)
            l = self.currentProject.Concordance.Lookup(self.myLine)
            n = l.ElementCount
            if n:
                ## EDIT -> STEG2(NY)
                ## EDIT -> STEG2(Punkt2)
                self.coordCtl.CursorStyle |= CursorStyle.RubberLine
                c = l.ElementAt(n - 1)
                if self.coordCtl.ResultCoordinateSystem:
                    ## EDIT -> STEG3(Ny linje)
                    ## EDIT -> STEG3(Punkt2)
                    self.coordCtl.AnchorPoint = MousePosition(self.coordCtl.ClickWindow, c.Position, self.coordCtl.ResultCoordinateSystem)
                else:
                    self.coordCtl.AnchorPoint = None
            else:
                self.coordCtl.CursorStyle &= ~CursorStyle.RubberLine


    def ModelEvents_EntityDeleted(self, obj, eArgs):
        serial = eArgs.EntitySerialNumber
        # check if current line is being deleted
        if serial == self.myLine:
            self.myLine = 0
            self.coordCtl.CursorStyle &= ~CursorStyle.RubberLine
            self.coordCtl.AnchorPoint = None
        pass

    def CancelClicked(self, thisCmd, args):

        #OBS! NOLLA ALLA GLOBALA VARIABLER!!!!
        self.cordChanged = 0

        self.lutningInst = None
        self.baringInst = None
        self.lutBarStatus = "OK"


        #save default name/layer
        if self.myLine:
            settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
            settings.CreateLinestringSettings.Name = self.nameCtl.SelectedName
            settings.CreateLinestringSettings.Layer = self.Layers.SelectedSerialNumber
        thisCmd.CloseUICommand()

    def NewClicked(self, thisCmd, e):
        self.myLine = 0
        self.coordCtl.CursorStyle &= ~self.coordCtl.CursorStyle.RubberLine
        self.coordCtl.AnchorPoint = None
        if self.nameCtl.AutoTabExtSkip:
            Keyboard.Focus(self.coordCtl)
        else:
            Keyboard.Focus(self.nameCtl)

    def OkClicked(self, thisCmd, e):
        settingsOk = True

        #Kontrollerar att inställningar är OK
        if self.lutObj.Text == "" or self.barObj.Text == "":
            settingsOk = False

        try:
            pallangdFloat = float(self.palLan.Text)
        except Exception as e:
            pallangdFloat = None

        #Körs om inställningar är OK
        if isinstance(pallangdFloat, float) and settingsOk == True:
            #Kontrollerar om man klickat i grafiken(i närheten av pålarna)
            if self.cordChanged == 0:
                VceMessageBox.ShowWarning("Klicka i närheten av pålarna")
                pass
            else:
                # inställnigar är OK
                # Kontroll av inställningar och sätter globala variabler
                # Används längre fram i macrot
                self.lutningInst = self.lutObj.Text
                self.baringInst = self.barObj.Text

                # Macrot körs
                listaMedPunkter = self.createListSelectedObjects()

                if len(listaMedPunkter) == 0:
                    self.lutningStatus = "OK"
                    VceMessageBox.ShowWarning("Felaktiga värden eller inställningar\nKontrollera lutning och/eller bäring")
                else:
                    for pointSelect in listaMedPunkter:

                        riktPunkt = self.newPtWithInclBearLangd(pointSelect, pointSelect.Des1, pointSelect.Des2, pallangdFloat)

                        if riktPunkt == "ERR":
                            self.lutningStatus = "OK"
                            VceMessageBox.ShowWarning("Felaktiga värden eller inställningar\nKontrollera lutning och/eller bäring")
                            break


                        self.calcDirectionLine(pointSelect,riktPunkt)
        else:
            VceMessageBox.ShowWarning("Kontrollera dina värden under inställningar")

        if isinstance(pallangdFloat, float) and settingsOk == True and self.cordChanged == 1:

            #OBS! NOLLA ALLA GLOBALA VÄRDEN!!!!
            self.cordChanged = 0

            self.lutningInst = None
            self.baringInst = None
            self.lutBarStatus = "OK"

            #save default name/layer
            if self.myLine:
                settings = ConstructionCommandsSettings.ProvideObject(self.currentProject)
                settings.CreateLinestringSettings.Name = self.nameCtl.SelectedName
                settings.CreateLinestringSettings.Layer = self.Layers.SelectedSerialNumber
            thisCmd.CloseUICommand()


    # TA BORT?
    def calcDirectionPoint(self, ptSel):

        riktPunkt = types.SimpleNamespace()

        riktPunkt.X = float(ptSel.X)+100
        riktPunkt.Y = float(ptSel.Y)+100
        riktPunkt.Z = float(ptSel.Z)+100
        riktPunkt.Namn = str(ptSel.Namn)+"-rikt"

        return riktPunkt




    def calcDirectionLine(self, original, rikt):

            self.createLine(None, original)
            self.createLine(None, rikt)
            self.NewClicked(None, None)








    def newPtWithInclBearLangd(self, pt, inclination, bearing, lutLangd):

        #Nollställer global variabel lutningstatus för kontroll
        self.lutBarStatus = "OK"


        #gör om LUTNING till grader
        if self.lutningInst == "Lutningsförhållande":
            #gör om lutningsförhållande till grader
            # Check if ":" or "/" is used as the divider
            inclination = str(inclination)
            if ":" in inclination:
                a, b = map(float, inclination.split(":"))
                # Calculate the slope ratio as a float
                inclination = a / b

                # Calculate the angle in degrees
                radians = math.atan(inclination)
                degrees = math.degrees(radians)

                inclination_degrees = float(degrees)
            elif "/" in inclination:
                a, b = map(float, inclination.split("/"))
                # Calculate the slope ratio as a float
                inclination = a / b

                # Calculate the angle in degrees
                radians = math.atan(inclination)
                degrees = math.degrees(radians)

                inclination_degrees = float(degrees)
            else:
                self.lutBarStatus = "ERR"

        if self.lutningInst == "Lutningsvinklar grader":
            #gör INGET
            try:
                inclination_degrees = float(inclination)
            except Exception as e:
                self.lutBarStatus = "ERR"

        if self.lutningInst == "Lutningsvinklar gon":
            #gör om Lutningsvinklar grader till grader
            try:
                inclination_degrees = float(inclination)*(360/400)
            except Exception as e:
                self.lutBarStatus = "ERR"

        if self.lutningInst == "Lutning procent":
            #gör om Lutning procent grader till grader
            try:
                inclination = str(inclination)
                if "%" in inclination:
                    inclination = inclination.replace("%","")
                    inclination = inclination.replace(" ","")

                inclination = float(inclination)
                # Convert the percent slope to a decimal
                slope_decimal = inclination / 100

                # Calculate the angle in degrees using the arctan function
                degrees = math.degrees(math.atan(slope_decimal))

                inclination_degrees = float(degrees)

            except Exception as e:
                self.lutBarStatus = "ERR"


        #gör om BARING till grader
        if self.baringInst == "Grader":
            #gör INGET
            try:
                bearing_degrees = float(bearing)
            except Exception as e:
                self.lutBarStatus = "ERR"

        if self.baringInst == "Gon":
            #görom till grader
            try:
                bearing_degrees = float(bearing)*(360/400)
            except Exception as e:
                self.lutBarStatus = "ERR"


        #KONTROLLERAR OM NÅGOT GICK SNETT.
        if self.lutBarStatus == "ERR":
            return "ERR"
        #KÖR VIDARE OM DET FUNKAR BRA
        else:
            newCalcPoint = types.SimpleNamespace()

            pt.N = pt.Y
            pt.E = pt.X
            # p3 förstx
            N1 = float(pt.N)  # Northing in meters
            E1 = float(pt.E)  # Easting in meters
            Z1 = float(pt.Z)
            NewN = 0
            NewE = 0


            inclination_degrees = abs(inclination_degrees)

            # VceMessageBox.Show(str(inclination_degrees))

            incl_trig = 90-inclination_degrees

            # VceMessageBox.Show(str(incl_trig)+"-inkl trig")
            inc_float = float(incl_trig)
            if inc_float > 0:
               pass
            elif inc_float == 0:
               pass
            else:
               incl_trig = abs(inc_float)


            incl_trig_rad = math.radians(incl_trig)

            deltaHor = (math.sin(incl_trig_rad))*lutLangd
            deltaVert = (math.cos(incl_trig_rad))*lutLangd

            NewZ=Z1-deltaVert





            if 0 <= bearing_degrees < 90:
                bear_trig_rad = math.radians(bearing_degrees)
                plusN = (math.cos(bear_trig_rad))*deltaHor
                plusE = (math.sin(bear_trig_rad))*deltaHor
                NewN = N1+float(plusN)
                NewE = E1+float(plusE)
            elif 90 <= bearing_degrees < 180:

                bear_trig = float(bearing_degrees)-90
                bear_trig_rad = math.radians(bear_trig)
                negN = (math.sin(bear_trig_rad))*deltaHor
                plusE = (math.cos(bear_trig_rad))*deltaHor
                NewN = N1-float(negN)
                NewE = E1+float(plusE)
            elif 180 <= bearing_degrees < 270:

                bear_trig = float(bearing_degrees)-180
                bear_trig_rad = math.radians(bear_trig)
                negN = (math.cos(bear_trig_rad))*deltaHor
                negE = (math.sin(bear_trig_rad))*deltaHor
                NewN = N1-float(negN)
                NewE = E1-float(negE)
            elif 270 <= bearing_degrees <= 360:

                bear_trig = float(bearing_degrees)-270
                bear_trig_rad = math.radians(bear_trig)
                plusN = (math.sin(bear_trig_rad))*deltaHor
                negE = (math.cos(bear_trig_rad))*deltaHor
                NewN = N1+float(plusN)
                NewE = E1-float(negE)
            else:
                pass




            newCalcPoint.Y = NewN
            newCalcPoint.X = NewE


            newCalcPoint.N = NewN
            newCalcPoint.E = NewE
            newCalcPoint.Z = NewZ
            newCalcPoint.Code = pt.Code

            newCalcPoint.Namn = pt.Namn+"-rkt"
            newCalcPoint.Des1 = ""
            newCalcPoint.Des2 = ""


            return newCalcPoint





    def createLine(self, thisCmd, e):


        punktInfo = e


        ctrl = self.coordCtl

        coord = ctrl.Coordinate

        self.coordCtl.StatusMessage = ""
        # tell undo manager we are starting to do something

        #MACARONI
        #self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)


        try:
            # VceMessageBox.ShowWarning("createLine TRY")
            # the "with" statement will unroll any changes if something goes
            # wrong
            l = None
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                crd = ctrl.Coordinate
                # string1 = str(crd)

                if hasattr(e, 'X'):

                    crd.X = float(punktInfo.X)
                    crd.Y = float(punktInfo.Y)
                    crd.Z = float(punktInfo.Z)

                e = self.elevCtl.Elevation
                if not self.nameCtl.AutoTabExtSkip and not Double.IsNaN(e):
                    crd.Z = e
                    self.elevCtl.Clear() # clear elevation after use
                if not self.myLine:
                    # STEG1 - Ny linje
                    # Häör skapas linjen med Point3D
                    #--------------------------

                    #now create a linestring
                    container = ViewHelper.GetContainerForView(ctrl.ClickWindow) # usually WorldView
                    l = container.Add(clr.GetClrType(Linestring))
                    self.myLine = l.SerialNumber # it's safer to hold onto serial numbers instead of the actual object
                    e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
                    e.Position = crd
                    l.AppendElement(e)
                else:
                    # STEG2 - Punk2 på linke

                    l = self.currentProject.Concordance.Lookup(self.myLine)
                    if l:
                        # check if dup location
                        n = l.ElementCount
                        if n:
                            c = l.ElementAt(n - 1)
                            r = Point3D.IsDuplicate2D(clr.Reference[Point3D](crd),clr.Reference[Point3D](c.Position))
                            if r:
                                # if coordinate same, just update (to get new
                                # elevation)
                                c.Position = crd
                                l.ReplaceElementAt(c, n - 1)
                            else:
                                if Control.ModifierKeys & Keys.Control:
                                    e = ElementFactory.Create(clr.GetClrType(ITangentArcSegment), clr.GetClrType(IXYZLocation))
                                    e.TangentType = ArcTangentType.TangentTangent
                                else:
                                    e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
                                e.Position = crd
                                l.AppendElement(e)

                #set name/layer each time to get changes
                # xxxxx Byt så namnet på linjen är samma som punkten
                l.Name = punktInfo.Namn
                sn = self.Layers.SelectedSerialNumber
                if sn:
                    l.Layer = sn

                # you MUST commit the change to DB or they will all be un-done
                # (roll-back)
                # (inside the "with" scope)
                failGuard.Commit()
        finally:
            pass
            #MACARONI
            #self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.coordCtl)
        return

    def createListSelectedObjects(self):
        #Selecting alla points
        selectedPoints = []
        i = 0
        str_i = "0"
        for o in self.objs.SelectedMembers(self.currentProject):

            if isinstance(o, self.pType):

                if str(o.Position.X) == "nan" or str(o.Position.Y) == "nan" or str(o.Position.Z) == "nan" or o.FullDescription1 == "" or o.FullDescription2 == "":
                    pass
                else:

                    punktInfo = types.SimpleNamespace()

                    #Adds id to array
                    if isinstance(o, self.pType):

                        punktInfo.Namn = "Pale-"+str(o.PointID)


                    #Adds coordinates
                    if isinstance(o, self.ptType):
                        # VceMessageBox.ShowWarning("XYZ")
                        punktInfo.X = str(o.Position.X)
                        punktInfo.Y = str(o.Position.Y)
                        punktInfo.Z = str(o.Position.Z)


                    #Adds featurecode to array
                    if isinstance(o, self.fcType):
                        punktInfo.Code = str(o.FeatureCode)



                    #Adds Description1 to array
                    if hasattr(o, 'Description1'):
                        punktInfo.Des1 = str(o.FullDescription1)



                    #Adds Description1 to array
                    if hasattr(o, 'Description2'):
                        punktInfo.Des2 = str(o.FullDescription2)



                    selectedPoints.insert(i, punktInfo)
                    i = i + 1

            else:
                pass




        return selectedPoints



#????????? TA BORT ?????????
    def resetLineArray(self):
        self.lineArray = []
        for i in range(75):
            self.lineArray.append(" ")


    def splitToArray(self, str):
        strArray = list(str)
        return strArray


    def get_random_string(self, length):
        letters = string.ascii_lowercase
        result_str = ''.join(random.choice(letters) for i in range(length))
        return result_str
